package ui;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;

import com.smardec.mousegestures.MouseGestures;
import com.smardec.mousegestures.MouseGesturesListener;

import core.Mapper;

public class MapperUIGesture extends JFrame{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private MapperPanel mapperPanel=null;
	private Mapper mapper=null;
	
	public MapperUIGesture(Mapper mapper){
		this.mapper=mapper;
		setTitle("Mapper");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLayout(new BorderLayout());
		setResizable(false);
		mapperPanel=new MapperPanel();
		mapperPanel.addMouseListener(mapperPanel);
		mapperPanel.addMouseMotionListener(mapperPanel);
		
		add(mapperPanel,BorderLayout.LINE_START);
		pack();
		setVisible(true);
	}
	
	
	class MapperPanel extends JPanel implements MouseGesturesListener,MouseMotionListener,MouseListener{
		
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private Point startPoint;
		private Point currentPoint;
		private double startValue;
		private String gesture;
		
		public MapperPanel(){
			setPreferredSize(mapper.getDimension());
			gesture="clean";
			
			MouseGestures mouseGestures = new MouseGestures();
			mouseGestures.setMouseButton(MouseEvent.BUTTON1_MASK);
			
			mouseGestures.addMouseGesturesListener(this);
			mouseGestures.start();
		}
		protected void paintComponent(Graphics g){
			Graphics2D g2d=(Graphics2D)g;
			if(gesture.equals("geste")){
				g2d.setColor(Color.BLUE);
				g2d.setStroke(new BasicStroke(2.0f));
				g2d.drawLine(startPoint.x,startPoint.y,currentPoint.x,currentPoint.y);
			}else{
				g2d.drawImage(mapper.getImage(),null,this);
				if(gesture.equals("clean")){gesture="geste";}
			}
			
			
		}
		
		public void mouseDragged(MouseEvent e){
			if(gesture.equals("geste")){
				startPoint=currentPoint;
				currentPoint=e.getPoint();
				repaint();
			}else if(gesture.equals("zoom")){
				double newZoom = Math.rint(startValue*100+(e.getX()-startPoint.getX())/2)/100d;
				if(Math.sqrt(Math.pow(mapper.getZoom()-newZoom, 2))>.1){mapper.setZoom(newZoom);repaint();}
			}
		}
		public void mouseMoved(MouseEvent e){}
		public void mouseClicked(MouseEvent e){}
		public void mouseEntered(MouseEvent e){}
		public void mouseExited(MouseEvent e){
		}
		public void mousePressed(MouseEvent e){
			if(gesture.equals("geste")){
				startPoint=currentPoint=e.getPoint();
				repaint();
			}else if(gesture.equals("zoom")){
				startPoint=e.getPoint();
				startValue=mapper.getZoom();
			}
		}
		public void mouseReleased(MouseEvent e){
			if(gesture.equals("geste")){gesture="clean";}
			repaint();
		}
		public void gestureMovementRecognized(String arg0) {}
		public void processGesture(String arg0) {
			if(arg0.equals("RDR")){
				gesture="zoom";
				repaint();
			}else{
				gesture="clean"; repaint();
			}
		}
	}
	
	class ButtonPanel extends JPanel{
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public ButtonPanel(String name){
			setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
			setBorder(BorderFactory.createTitledBorder(name));
		}
	}
}
