package ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Point2D;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JToggleButton;

import core.Mapper;

public class MapperUIModal extends JFrame{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private MapperPanel mapperPanel=null;
	private ControlPanel controlPanel=null;
	private Mapper mapper=null;
	
	ButtonPanel action;
	Zoom z;
	
	public MapperUIModal(Mapper mapper){
		this.mapper=mapper;
		setTitle("Mapper");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLayout(new BorderLayout());
		setResizable(false);
		mapperPanel=new MapperPanel();
		mapperPanel.addMouseListener(mapperPanel);
		mapperPanel.addMouseMotionListener(mapperPanel);
		
		controlPanel=new ControlPanel();
		add(mapperPanel,BorderLayout.LINE_START);
		add(controlPanel,BorderLayout.CENTER);
		pack();
		setVisible(true);
	}
	
	class ControlPanel extends JPanel{
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public ControlPanel(){
			setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
			setBorder(BorderFactory.createTitledBorder("Control"));
			
			z=new Zoom("zoom");
			action=new ButtonPanel("Actions:");
			action.add(z);
			
			add(action);
		}
	}
	
	
	class Zoom extends JToggleButton implements ActionListener{
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public Zoom(String string) {
			super(string);
		}

		public void actionPerformed(ActionEvent e) {
			mapper.setZoom(mapper.getZoom()+0.01);
			mapperPanel.repaint();
		}
	}
	
	
	class MapperPanel extends JPanel implements MouseMotionListener,MouseListener{
		
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private Point2D startPoint;
		private double startValue;
		
		public MapperPanel(){
			setPreferredSize(mapper.getDimension());
			setBackground(Color.blue);
		}
		protected void paintComponent(Graphics g){
			Graphics2D g2d=(Graphics2D)g;
			g2d.drawImage(mapper.getImage(),null,this);
		}
		
		public void mouseDragged(MouseEvent e){
			if(z.isSelected()){
				double newZoom = Math.rint(startValue*100+(e.getX()-startPoint.getX())/2)/100d;
				if(Math.sqrt(Math.pow(mapper.getZoom()-newZoom, 2))>.1){mapper.setZoom(newZoom);repaint();}
				
			}
		}
		public void mouseMoved(MouseEvent e){}
		public void mouseClicked(MouseEvent e){}
		public void mouseEntered(MouseEvent e){}
		public void mouseExited(MouseEvent e){
		}
		public void mousePressed(MouseEvent e){
			startPoint=e.getPoint();
			if(z.isSelected()){
				startValue=mapper.getZoom();
			}
		}
		public void mouseReleased(MouseEvent e){
			repaint();
		}
	}
	
	class ButtonPanel extends JPanel{
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public ButtonPanel(String name){
			setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
			setBorder(BorderFactory.createTitledBorder(name));
		}
	}
}
